package de.jugmuenster.sevendemo.coin;

/**
 * Java 7 Demo: Multi Catch
 */

class MainException extends Exception
{
    private static final long serialVersionUID = 1L;
}
class SubException extends MainException
{
    private static final long serialVersionUID = 1L;
}
class OffByOneException extends SubException
{
    private static final long serialVersionUID = 1L;
}

public class MultiCatch
{
    public static void main(String[] args)
    {
        showMultiCatch();
        try
        {
            showReThrow();
        }
        catch (MainException ex)
        {
            System.out.println(String.format("Caught '%s' exception after re-throw.",
                    ex.getClass().getSimpleName()));
        }
    }

    private static void showMultiCatch()
    {
        final int number = 47;

        try
        {
            if (number == 7)
            {
                throw new MainException();
            }
            if(number > 43)
            {
                throw new SubException();
            }
            if (Math.abs(number - 42) <= 1)
            {
                throw new OffByOneException();
            }
        }
        //order is important as usual
        catch (OffByOneException ex)
        {
            System.out.println("Off by one!");
        }
        catch (SubException | MainException ex)
        {
            System.out.println(String.format("Caught a '%s' exception.",
                    ex.getClass().getSimpleName()));
        }

    }


    
    private static void showReThrow() throws MainException
    {
        final int number = 47;

        try
        {
            if (number == 7)
            {
                throw new MainException();
            }
            if(number > 43)
            {
                throw new SubException();
            }
            if (Math.abs(number - 42) <= 1)
            {
                throw new OffByOneException();
            }
        }
        //generic exception handling possible without multiple catch statements
        catch (final Exception ex) //must be final
        {
            System.out.println(String.format("Caught a '%s' exception and will throw it...",
                    ex.getClass().getSimpleName()));
            throw ex;
        }
    }
}
